#include "math.h"
#include "limits.h"
#include "mex.h"

static union
{
  double d;
    struct
    {
#ifdef LITTLE_ENDIAN
      int j, i;
#else
      int i, j;
#endif
    } n;
} _eco;
#define EXP_A (1048576/M_LN2)
#define EXP_C 60801

#define EXP(y) (_eco.n.i = EXP_A*(y) + (1072693248 - EXP_C), _eco.d)


#define RANGE 600
#define N_RANGE -600
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    const mxArray *mxIn;
    double *xValues, *outArray, *inArray;
    double element;
    int i, nIn;

    if (nrhs!=1)
        mexErrMsgTxt("There must be one and only input.");
    if (nlhs>1) /* can be "zero", in which case Matlab allows one */
        mexErrMsgTxt("There must be one and only output.");

    /* get information about the input */
    mxIn = prhs[0];
    inArray = mxGetPr(mxIn);
    nIn = mxGetNumberOfElements(prhs[0]);

    /* create output the same size as the input */
    plhs[0] = mxCreateNumericArray(
            mxGetNumberOfDimensions(mxIn),
            mxGetDimensions(mxIn),
            mxDOUBLE_CLASS, mxREAL);
    outArray = mxGetPr(plhs[0]);

    /* EXP(.) every element of array in "Fortran order" */
    for(i=0; i < nIn; i++) {
        element = inArray[i];
        if (element > RANGE)
            outArray[i] = INFINITY;
        else if (element < N_RANGE)
            outArray[i] = 0;
        else
            outArray[i] = EXP(element);
    }

}

